@@ -70,7 +70,7 @@ void hkl_binoculars_axis_limits_free(HklBinocularsAxisLimits *self)
 HklBinocularsAxisLimits *hkl_binoculars_axis_limits_new(const ptrdiff_t *imin,
                                                         const ptrdiff_t *imax)
 {
-	HklBinocularsAxisLimits *self = malloc(sizeof(*self));
+	HklBinocularsAxisLimits *self = g_new(HklBinocularsAxisLimits, 1);
 
         if (NULL == imin){
                 self->imin = NoLimit();
@@ -115,14 +115,14 @@ static inline void hkl_binoculars_axis_init(HklBinocularsAxis *self,
 
 double *hkl_binoculars_axis_array(const HklBinocularsAxis *self)
 {
-	double *arr = malloc(6 * sizeof(*arr));
+	double *arr = g_new0(double, 6);
 
-	arr[0] = self->index;
-	arr[1] = axis_min(self);
-	arr[2] = axis_max(self);
-	arr[3] = self->resolution;
-	arr[4] = self->imin;
-	arr[5] = self->imax;
+        arr[0] = self->index;
+        arr[1] = axis_min(self);
+        arr[2] = axis_max(self);
+        arr[3] = self->resolution;
+        arr[4] = self->imin;
+        arr[5] = self->imax;
 
 	return arr;
 }
@@ -185,43 +185,6 @@ static inline int does_not_include(const darray_axis *axes,
 
 /* Space */
 
-HklBinocularsSpace *hkl_binoculars_space_new(size_t max_items, size_t n_axes)
-{
-	HklBinocularsSpace *self = malloc(sizeof(*self));
-
-	self->max_items = max_items;
-        darray_init(self->items);
-        darray_resize(self->items, max_items);
-        darray_init(self->axes);
-        darray_resize(self->axes, n_axes);
-
-	return self;
-}
-
-void hkl_binoculars_space_free(HklBinocularsSpace *self)
-{
-	darray_free(self->axes);
-        darray_free(self->items);
-	free(self);
-}
-
-void hkl_binoculars_space_fprintf(FILE *f, const HklBinocularsSpace *self)
-{
-        size_t masked;
-        HklBinocularsAxis *axis;
-
-	fprintf(f, "\nHklBinocularsSpace: %p", self);
-	fprintf(f, "\nn_indexes_0: %ld", darray_size(self->items));
-	fprintf(f, "\nn_axes: %ld", darray_size(self->axes));
-        darray_foreach(axis, self->axes){
-		fprintf(f, "\n");
-		hkl_binoculars_axis_fprintf(f, axis);
-	}
-
-        masked = self->max_items - darray_size(self->items);
-        fprintf(f, "\nmasked pixels: %ld (%f%%)", masked, (double)masked / self->max_items * 100);
-}
-
 static inline void hkl_binoculars_space_item_fprintf(FILE *f, const HklBinocularsSpaceItem *self)
 {
         fprintf(f, "item->indexes(%p) v: %ld %ld %ld, intensity: %d", &self->indexes_0[0],
@@ -296,7 +259,120 @@ static inline int item_in_the_limits(const HklBinocularsSpaceItem *item,
         return res;
 }
 
-/* the array is pre filled with the pixel coordinates */
+HklBinocularsSpace *hkl_binoculars_space_new(size_t max_items, size_t n_axes)
+{
+	HklBinocularsSpace *self = g_new(HklBinocularsSpace, 1);
+
+        self->max_items = max_items;
+        darray_init(self->items);
+        darray_resize(self->items, max_items);
+        darray_init(self->axes);
+        darray_resize(self->axes, n_axes);
+
+	return self;
+}
+
+void hkl_binoculars_space_free(HklBinocularsSpace *self)
+{
+	darray_free(self->axes);
+        darray_free(self->items);
+	free(self);
+}
+
+void hkl_binoculars_space_fprintf(FILE *f, const HklBinocularsSpace *self)
+{
+        size_t masked;
+        HklBinocularsAxis *axis;
+
+	fprintf(f, "\nHklBinocularsSpace: %p", self);
+	fprintf(f, "\nn_indexes_0: %ld", darray_size(self->items));
+        if(space_is_empty(self)){
+                fprintf(f, "\nempty");
+        } else {
+                fprintf(f, "\nn_axes: %ld", darray_size(self->axes));
+                darray_foreach(axis, self->axes){
+                        fprintf(f, "\n");
+                        hkl_binoculars_axis_fprintf(f, axis);
+                }
+
+                masked = self->max_items - darray_size(self->items);
+                fprintf(f, "\nmasked pixels: %ld (%f%%)", masked, (double)masked / self->max_items * 100);
+        }
+}
+
+/* angles */
+
+#define HKL_BINOCULARS_SPACE_ANGLES_IMPL(image_t)                       \
+        HKL_BINOCULARS_SPACE_ANGLES_DECL(image_t)                       \
+        {                                                               \
+                size_t i, j;                                            \
+                const char * names[] = {"delta", "gamma", "omega"};     \
+                const HklParameter *p;                                  \
+                double delta0;                                          \
+                double gamma0;                                          \
+                double omega0;                                          \
+                                                                        \
+                assert(ARRAY_SIZE(names) == darray_size(space->axes));  \
+                assert(ARRAY_SIZE(names) == n_resolutions);             \
+                assert(n_pixels == space->max_items);                   \
+                                                                        \
+                darray_size(space->items) = 0;                          \
+                                                                        \
+                const double *p_x = &pixels_coordinates[0 * n_pixels];  \
+                const double *p_y = &pixels_coordinates[1 * n_pixels];  \
+                const double *p_z = &pixels_coordinates[2 * n_pixels];  \
+                                                                        \
+                p = hkl_geometry_axis_get(geometry, "delta", NULL);    \
+                if(p){                                                  \
+                        delta0 = hkl_parameter_value_get(p, HKL_UNIT_USER); \
+                } else {                                                \
+                        delta0 = 0;                                      \
+                }                                                       \
+                p = hkl_geometry_axis_get(geometry, "gamma", NULL);    \
+                if(p){                                                  \
+                        gamma0 = hkl_parameter_value_get(p, HKL_UNIT_USER); \
+                } else {                                                \
+                        gamma0 = 0;                                     \
+                }                                                       \
+                p = hkl_geometry_axis_get(geometry, "omega", NULL);    \
+                if(p){                                                  \
+                        omega0 = hkl_parameter_value_get(p, HKL_UNIT_USER); \
+                } else {                                                \
+                        omega0 = 0;                                     \
+                }                                                       \
+                                                                        \
+                HklDetector *detector = hkl_detector_factory_new(HKL_DETECTOR_TYPE_0D); \
+                const HklQuaternion q = hkl_geometry_detector_rotation_get(geometry, detector); \
+                                                                        \
+                for(i=0;i<n_pixels;++i){                                \
+                        if(NULL == masked || 0 == masked[i]){           \
+                                HklBinocularsSpaceItem item;            \
+                                HklVector v = {{p_x[i], p_y[i], p_z[i]}}; \
+                                                                        \
+                                hkl_vector_rotated_quaternion(&v, &q);  \
+                                v.data[0] = atan(v.data[2]) / M_PI * 180.0 + delta0; \
+                                v.data[1] = atan(v.data[1]) / M_PI * 180.0 + gamma0; \
+                                v.data[2] = omega0;                     \
+                                                                        \
+                                for(j=0; j<ARRAY_SIZE(names); ++j){     \
+                                        item.indexes_0[j] = rint(v.data[j] / resolutions[j]); \
+                                }                                       \
+                                item.intensity = rint((double)image[i] * weight); \
+                                                                        \
+                                                                        \
+                                if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
+                                        darray_append(space->items, item); \
+                        }                                               \
+                }                                                       \
+                                                                        \
+                space_update_axes(space, names, n_pixels, resolutions); \
+                                                                        \
+                hkl_detector_free(detector);                            \
+        }
+
+HKL_BINOCULARS_SPACE_ANGLES_IMPL(int32_t);
+HKL_BINOCULARS_SPACE_ANGLES_IMPL(uint16_t);
+HKL_BINOCULARS_SPACE_ANGLES_IMPL(uint32_t);
 
 /* qparqper */
 
@@ -512,37 +588,33 @@ static inline ptrdiff_t compute_offset0(const darray_axis *axes)
 	return offset;
 }
 
-static inline HklBinocularsCube *empty_cube(const darray_axis *axes)
+static inline HklBinocularsCube *empty_cube_from_axes(const darray_axis *axes)
 {
-        HklBinocularsAxis *axis;
-        HklBinocularsCube *self = malloc(sizeof(HklBinocularsCube));
+        HklBinocularsCube *self = NULL;
 
-	/* compute the final cube dimensions and the index offset */
-        darray_init(self->axes);
-        if (NULL != axes){
+        if (0 != darray_size(*axes)){
+                HklBinocularsAxis *axis;
+
+                self = g_new(HklBinocularsCube, 1);
+                darray_init(self->axes);
                 darray_foreach(axis, *axes){
                         darray_append(self->axes, *axis);
                 }
+                self->offset0 = compute_offset0(&self->axes);
+                self->photons = NULL;
+                self->contributions = NULL;
         }
-        self->offset0 = compute_offset0(&self->axes);
-        self->photons = NULL;
-        self->contributions = NULL;
 
         return self;
 }
 
-static inline int cube_is_empty(const HklBinocularsCube *self)
-{
-        return 0 == darray_size(self->axes);
-}
-
 static inline size_t cube_size(const HklBinocularsCube *self)
 {
-	HklBinocularsAxis *axis;
-	size_t n = 1;
+        size_t n = 1;
+        HklBinocularsAxis *axis;
 
         darray_foreach(axis, self->axes){
-		n *= axis_size(axis);
+                n *= axis_size(axis);
         }
 
 	return n;
@@ -552,8 +624,8 @@ static inline size_t malloc_cube(HklBinocularsCube *self)
 {
         size_t n = cube_size(self);
 
-	self->photons = malloc(n * sizeof(*self->photons));
-	self->contributions = malloc(n * sizeof(*self->contributions));
+        self->photons = malloc(n * sizeof(*self->photons));
+        self->contributions = malloc(n * sizeof(*self->contributions));
 
         return n;
 }
@@ -562,31 +634,20 @@ static inline size_t calloc_cube(HklBinocularsCube *self)
 {
         size_t n = cube_size(self);
 
-	self->photons = calloc(n, sizeof(*self->photons));
-	self->contributions = calloc(n, sizeof(*self->contributions));
+        self->photons = calloc(n, sizeof(*self->photons));
+        self->contributions = calloc(n, sizeof(*self->contributions));
 
         return n;
 }
 
-void hkl_binoculars_cube_free(HklBinocularsCube *self)
-{
-        /* fprintf(stdout, "\nHklBinocularsCube: deallocated %p\n", self); */
-
-	free(self->contributions);
-	free(self->photons);
-	darray_free(self->axes);
-	free(self);
-}
-
-
 /* Using this method the Cube has already the right dimensions, we
  * just add the Space data into it. */
 static inline void add_non_empty_space(HklBinocularsCube *cube,
                                        const HklBinocularsSpace *space)
 {
-	size_t i;
+        size_t i;
         size_t n_axes = darray_size(cube->axes);
-	ptrdiff_t lens[n_axes];
+        ptrdiff_t lens[n_axes];
         HklBinocularsSpaceItem *item;
 
         assert(n_axes == darray_size(space->axes));
@@ -594,9 +655,9 @@ static inline void add_non_empty_space(HklBinocularsCube *cube,
         /* compute the lens */
         lens[0] = 1;
 
-	for(i=1; i<n_axes; ++i){
+        for(i=1; i<n_axes; ++i){
                 lens[i] = lens[i - 1] * axis_size(&darray_item(cube->axes, n_axes - i));
-	}
+        }
 
         darray_foreach(item, space->items){
                 ptrdiff_t w = -cube->offset0;
@@ -612,24 +673,52 @@ static inline void add_non_empty_space(HklBinocularsCube *cube,
         }
 }
 
+static inline int cube_is_empty(const HklBinocularsCube *self)
+{
+        return 0 == darray_size(self->axes);
+}
+
+HklBinocularsCube *hkl_binoculars_cube_new_empty(void)
+{
+        HklBinocularsCube *self = g_new(HklBinocularsCube, 1);
+
+        darray_init(self->axes);
+        self->offset0 = 0;
+        self->photons = NULL;
+        self->contributions = NULL;
+
+        return self;
+}
+
+void hkl_binoculars_cube_free(HklBinocularsCube *self)
+{
+        free(self->contributions);
+        free(self->photons);
+        darray_free(self->axes);
+        free(self);
+}
+
 void hkl_binoculars_cube_fprintf(FILE *f, const HklBinocularsCube *self)
 {
         HklBinocularsAxis *axis;
 
-	fprintf(f, "HklBinocularsCube: %p\n", self);
-	fprintf(f, "n_axes: %ld", darray_size(self->axes));
+	fprintf(f, "HklBinocularsCube: %p", self);
+
+        fprintf(f, "\nn_axes: %ld", darray_size(self->axes));
         darray_foreach(axis, self->axes){
-		fprintf(f, "\n");
-		hkl_binoculars_axis_fprintf(f, axis);
-	}
-	fprintf(f, "\nphotons: %p", self->photons);
-	fprintf(f, "\ncontributions: %p", self->contributions);
+                fprintf(f, "\n");
+                hkl_binoculars_axis_fprintf(f, axis);
+        }
+        fprintf(f, "\nphotons: %p", self->photons);
+        fprintf(f, "\ncontributions: %p", self->contributions);
 }
 
-void hkl_binoculars_cube_dims(const HklBinocularsCube *self, size_t ndim, size_t *dims)
+void hkl_binoculars_cube_dims(const HklBinocularsCube *self, size_t ndims, size_t *dims)
 {
         HklBinocularsAxis *axis;
 
+        assert(ndims == darray_size(self->axes));
+
         darray_foreach(axis, self->axes){
                 *(dims++) = axis_size(axis);
         }
@@ -651,71 +740,65 @@ HklBinocularsCube *hkl_binoculars_cube_new(size_t n_spaces,
                                            const HklBinocularsSpace *const *spaces)
 {
         size_t i, i0;
-        HklBinocularsCube *self;
+        HklBinocularsCube *self = NULL;
 
         i0 = find_first_non_empty_space_index(n_spaces, spaces);
 
-        if (i0 == n_spaces){ /* all spaces are empty */
-                self = empty_cube(NULL);
-        } else {
-                self = empty_cube(&spaces[i0]->axes);
+        if(i0 < n_spaces){
+                self = empty_cube_from_axes(&spaces[i0]->axes);
 
-                /* compute the final cube dimensions and the index offset */
-                for(i=i0; i<n_spaces; ++i){
-                        const HklBinocularsSpace *space = spaces[i];
+                if(NULL != self){
+                        /* compute the final cube dimensions and the index offset */
+                        for(i=i0; i<n_spaces; ++i){
+                                const HklBinocularsSpace *space = spaces[i];
 
-                        if(space_is_empty(space))
-                                continue;
+                                if(space_is_empty(space))
+                                        continue;
 
-                        merge_axes(&self->axes, &space->axes);
-                }
-                self->offset0 = compute_offset0(&self->axes);
+                                merge_axes(&self->axes, &space->axes);
+                        }
+                        self->offset0 = compute_offset0(&self->axes);
 
-                /* allocated the final cube photons and contributions */
-                calloc_cube(self);
+                        /* allocated the final cube photons and contributions */
+                        calloc_cube(self);
 
-                /* add all the spaces */
-                for(i=i0; i<n_spaces; ++i){
-                        const HklBinocularsSpace *space = spaces[i];
+                        /* add all the spaces */
+                        for(i=i0; i<n_spaces; ++i){
+                                const HklBinocularsSpace *space = spaces[i];
 
-                        if(space_is_empty(space))
-                                continue;
+                                if(space_is_empty(space))
+                                        continue;
 
-                        add_non_empty_space(self, space);
+                                add_non_empty_space(self, space);
+                        }
                 }
         }
 
 	return self;
 }
 
-HklBinocularsCube *hkl_binoculars_cube_new_empty(void)
-{
-        return empty_cube(NULL);
-}
-
 HklBinocularsCube *hkl_binoculars_cube_new_empty_from_cube(const HklBinocularsCube *cube)
 {
-	HklBinocularsCube *self = empty_cube(&cube->axes);
-
-	/* allocated the final cube */
-        calloc_cube(self);
+	HklBinocularsCube *self = empty_cube_from_axes(&cube->axes);
+        if(NULL != self)
+                calloc_cube(self);
 
 	return self;
 }
 
 HklBinocularsCube *hkl_binoculars_cube_new_from_space(const HklBinocularsSpace *space)
 {
-	HklBinocularsCube *self;
+	HklBinocularsCube *self = NULL;
 
         if(space_is_empty(space)){
-                self = empty_cube(NULL);
+                self =  hkl_binoculars_cube_new_empty();
         } else {
-                self = empty_cube(&space->axes);
+                self = empty_cube_from_axes(&space->axes);
+                if(NULL != self){
+                        calloc_cube(self);
 
-                /* allocated the final cube */
-                calloc_cube(self);
-
-                add_non_empty_space(self, space);
+                        add_non_empty_space(self, space);
+                }
         }
 
         return self;
@@ -724,17 +807,19 @@ HklBinocularsCube *hkl_binoculars_cube_new_from_space(const HklBinocularsSpace *
 HklBinocularsCube *hkl_binoculars_cube_new_copy(const HklBinocularsCube *src)
 {
         size_t n;
-	HklBinocularsCube *self = empty_cube(&src->axes);
+	HklBinocularsCube *self = empty_cube_from_axes(&src->axes);
 
-	/* allocate the final cube */
-        n = malloc_cube(self);
+        if(NULL != self){
+                /* allocate the final cube */
+                n = malloc_cube(self);
 
-        /* copy the data */
-        memcpy(self->photons, src->photons, n * sizeof(*self->photons));
-        memcpy(self->contributions, src->contributions, n * sizeof(*self->contributions));
+                /* copy the data */
+                if(self->photons)
+                        memcpy(self->photons, src->photons, n * sizeof(*self->photons));
+                if(self->contributions)
+                        memcpy(self->contributions, src->contributions, n * sizeof(*self->contributions));
+        }
 
-        hkl_binoculars_cube_fprintf(stdout, src);
-        hkl_binoculars_cube_fprintf(stdout, self);
         return self;
 }
 
@@ -765,7 +850,6 @@ static inline void cube_add_cube_2(HklBinocularsCube *self,
         }
 }
 
-
 static inline void cube_add_cube_3(HklBinocularsCube *self,
                                    const HklBinocularsCube *other)
 {
@@ -801,9 +885,6 @@ static inline void cube_add_cube_3(HklBinocularsCube *self,
 static inline void cube_add_cube(HklBinocularsCube *self,
                                  const HklBinocularsCube *other)
 {
-        if ((NULL == other->photons) || (NULL == other->contributions))
-                return;
-
         assert(darray_size(self->axes) == darray_size(other->axes));
 
         switch(darray_size(self->axes)){
@@ -827,15 +908,14 @@ static inline void compute_strides(const darray_axis *axes, size_t strides[], si
 HklBinocularsCube *hkl_binoculars_cube_new_merge(const HklBinocularsCube *cube1,
                                                  const HklBinocularsCube *cube2)
 {
-        HklBinocularsCube *self = empty_cube(&cube1->axes);
-
-        merge_axes(&self->axes, &cube2->axes);
-        self->offset0 = compute_offset0(&self->axes);
-        calloc_cube(self);
-
-        cube_add_cube(self, cube1);
-        cube_add_cube(self, cube2);
+        HklBinocularsCube *self = empty_cube_from_axes(&cube1->axes);
+        if(NULL != self){
+                merge_axes(&self->axes, &cube2->axes);
+                calloc_cube(self);
 
+                cube_add_cube(self, cube1);
+                cube_add_cube(self, cube2);
+        }
         return self;
 }
 
@@ -866,42 +946,40 @@ static inline void switch_content(HklBinocularsCube *self,
 void hkl_binoculars_cube_add_space(HklBinocularsCube *self,
                                    const HklBinocularsSpace *space)
 {
-        HklBinocularsCube *cube;
-
 #ifdef DEBUG
         fprintf(stdout, "\nENTERING hkl_binoculars_cube_add_space:\n");
         hkl_binoculars_cube_fprintf(stdout, self);
         hkl_binoculars_space_fprintf(stdout, space);
 #endif
-        if(cube_is_empty(self)){
+        /* check the compatibility of the cube and the space. */
+        if (1 != space_is_empty(space)){
+                if (does_not_include(&self->axes, &space->axes)){
 #ifdef DEBUG
-                fprintf(stdout, "\nCreate a new empty cube");
+                        fprintf(stdout, "\nthe Cube does not contain the space, so create a new cube.");
 #endif
-
-                cube = hkl_binoculars_cube_new_from_space(space);
-                switch_content(self, cube);
-                hkl_binoculars_cube_free(cube);
-        }else{
-                /* check the compatibility of the cube and the space. */
-                if (!space_is_empty(space)){
-                        if (does_not_include(&self->axes, &space->axes)){
-#ifdef DEBUG
-                                fprintf(stdout, "\nthe Cube does not contain the space, so create a new cube.");
-#endif
-                                HklBinocularsCube *cube = empty_cube(&self->axes);
-
-                                merge_axes(&cube->axes, &space->axes); /* circonscript */
-                                cube->offset0 = compute_offset0(&cube->axes);
-                                calloc_cube(cube);
-
-                                cube_add_cube(cube, self);
-
-                                switch_content(self, cube);
-                                hkl_binoculars_cube_free(cube);
+                        if(0 != darray_size(self->axes)){ /* self cube is not empty */
+                                HklBinocularsCube *cube = empty_cube_from_axes(&self->axes);
+                                if(NULL != cube){
+                                        merge_axes(&cube->axes, &space->axes); /* circonscript */
+                                        cube->offset0 = compute_offset0(&cube->axes);
+                                        calloc_cube(cube);
+                                        cube_add_cube(cube, self);
+                                        switch_content(self, cube);
+                                        hkl_binoculars_cube_free(cube);
+                                }
+                        } else { /* self cube is empty */
+                                HklBinocularsCube *cube =  empty_cube_from_axes(&space->axes);
+                                if(NULL != cube){
+                                        cube->offset0 = compute_offset0(&cube->axes);
+                                        calloc_cube(cube);
+                                        switch_content(self, cube);
+                                        hkl_binoculars_cube_free(cube);
+                                }
                         }
-                        add_non_empty_space(self, space);
                 }
+                add_non_empty_space(self, space);
         }
+
 #ifdef DEBUG
         fprintf(stdout, "\n");
         hkl_binoculars_cube_fprintf(stdout, self);
