@@ -1,9 +1,12 @@
+{-# LANGUAGE DeriveAnyClass            #-}
+{-# LANGUAGE DeriveGeneric             #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE ForeignFunctionInterface  #-}
 {-# LANGUAGE GADTs                     #-}
 {-# LANGUAGE UnicodeSyntax             #-}
+
 {-
-    Copyright  : Copyright (C) 2014-2021 Synchrotron SOLEIL
+    Copyright  : Copyright (C) 2014-, 20222022 Synchrotron SOLEIL
                                          L'Orme des Merisiers Saint-Aubin
                                          BP 48 91192 GIF-sur-YVETTE CEDEX
     License    : GPL3+
@@ -63,8 +66,7 @@ import           Bindings.HDF5.Core              (HSize (HSize),
                                                   iterOrderCode)
 import           Bindings.HDF5.Dataset           (Dataset, closeDataset,
                                                   createDataset,
-                                                  getDatasetSpace,
-                                                  getDatasetType, openDataset,
+                                                  getDatasetSpace, openDataset,
                                                   readDataset, readDatasetInto,
                                                   writeDataset)
 import           Bindings.HDF5.Dataspace         (Dataspace,
@@ -75,7 +77,6 @@ import           Bindings.HDF5.Dataspace         (Dataspace,
                                                   getSimpleDataspaceExtentNDims,
                                                   getSimpleDataspaceExtentNPoints,
                                                   selectHyperslab, selectNone)
-import           Bindings.HDF5.Datatype          (getTypeClass, getTypeSize)
 import           Bindings.HDF5.Datatype.Internal (NativeType, hdfTypeOf1,
                                                   nativeTypeOf)
 import           Bindings.HDF5.Error             (withErrorCheck_)
@@ -96,6 +97,8 @@ import           Bindings.HDF5.Raw               (H5L_info_t, HErr_t (HErr_t),
                                                   h5s_ALL)
 import           Control.Exception               (Exception, bracket, throwIO)
 import           Control.Monad.Extra             (fromMaybeM)
+import           Data.Aeson                      (FromJSON (..), ToJSON (..),
+                                                  Value (..))
 import           Data.Array.Repa                 (Array, Shape, extent,
                                                   linearIndex, listOfShape,
                                                   size)
@@ -104,6 +107,7 @@ import           Data.ByteString.Char8           (ByteString, pack, packCString,
                                                   unpack)
 import           Data.IORef                      (modifyIORef', newIORef,
                                                   readIORef)
+import           Data.Text.Encoding              (decodeUtf8, encodeUtf8)
 import           Data.Vector.Storable            (Storable, Vector, freeze,
                                                   head, unsafeFromForeignPtr0)
 import           Data.Vector.Storable.Mutable    (new)
@@ -121,6 +125,7 @@ import           Foreign.StablePtr               (StablePtr, castPtrToStablePtr,
                                                   deRefStablePtr, freeStablePtr,
                                                   newStablePtr)
 import           GHC.Base                        (Alternative (..))
+import           GHC.Generics                    (Generic)
 import           Numeric.LinearAlgebra           (Matrix, reshape)
 
 import           Hkl.Detector
@@ -246,16 +251,7 @@ datasetShape d = withDataspace (getDatasetSpace d) getSimpleDataspaceExtent
 --  DataSet
 
 openDataset' :: Location l => l -> ByteString -> Maybe DAPL -> IO Dataset
-openDataset' loc n ml = do
-  ds <- openDataset loc n ml
-  s <- getDatasetSpace ds
-  np <- getSimpleDataspaceExtentNPoints s
-  e <- getSimpleDataspaceExtent s
-  t <- getDatasetType ds
-  c <- getTypeClass t
-  es <- getTypeSize t
-  -- print (n, c, es, np, e)
-  return ds
+openDataset' = openDataset
 
 withDataset :: IO Dataset -> (Dataset -> IO r) -> IO r
 withDataset a = bracket a closeDataset
@@ -382,7 +378,16 @@ data Hdf5Path sh e
   | H5DatasetPath ByteString
   | H5DatasetPathAttr (ByteString, ByteString)
   | H5Or (Hdf5Path sh e) (Hdf5Path sh e)
-    deriving (Show)
+    deriving (Eq, Generic, Show, FromJSON, ToJSON)
+
+instance ToJSON ByteString where
+    toJSON = String . decodeUtf8
+    {-# INLINE toJSON #-}
+
+instance FromJSON ByteString where
+    parseJSON (String t) = pure . encodeUtf8 $ t
+    parseJSON _          = GHC.Base.empty
+    {-# INLINE parseJSON #-}
 
 hdf5p :: Hdf5Path sh e -> Hdf5Path sh e
 hdf5p = H5RootPath
