@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2020 Synchrotron SOLEIL
+ * Copyright (C) 2003-2020, 2022 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -86,7 +86,7 @@ static struct HklHolderConfig *hkl_holder_config_new(void)
 {
 	struct HklHolderConfig *self;
 
-	self = HKL_MALLOC(struct HklHolderConfig);
+	self = g_new(struct HklHolderConfig, 1);
 
 	self->gc = 1;
 	self->idx = NULL;
@@ -124,7 +124,7 @@ static void hkl_holder_config_unref(struct HklHolderConfig *self)
 static HklHolder *hkl_holder_new(HklGeometry *geometry)
 {
 	static HklQuaternion q0 = {{1, 0, 0, 0}};
-	HklHolder *self = HKL_MALLOC(HklHolder);
+	HklHolder *self = g_new(HklHolder, 1);
 
 	self->config = hkl_holder_config_new();
 	self->geometry = geometry;
@@ -135,7 +135,7 @@ static HklHolder *hkl_holder_new(HklGeometry *geometry)
 
 static HklHolder *hkl_holder_new_copy(HklHolder *src, HklGeometry *geometry)
 {
-	HklHolder *self = HKL_MALLOC(HklHolder);
+	HklHolder *self = g_new(HklHolder, 1);
 
 	self->config = hkl_holder_config_ref(src->config);
 	self->geometry = geometry;
@@ -287,11 +287,9 @@ const HklGeometryOperations hkl_geometry_operations_defaults = { HKL_GEOMETRY_OP
 HklGeometry *hkl_geometry_new(const HklFactory *factory,
 			      const HklGeometryOperations *ops)
 {
-	HklGeometry *g = NULL;
+	HklGeometry *g = g_new(HklGeometry, 1);
 
-	g = HKL_MALLOC(HklGeometry);
 	g->ops = ops;
-
 	g->factory = factory;
 	hkl_source_init(&g->source, 1.54, 1, 0, 0);
 	darray_init(g->axes);
@@ -310,12 +308,10 @@ HklGeometry *hkl_geometry_new(const HklFactory *factory,
  **/
 HklGeometry *hkl_geometry_new_copy(const HklGeometry *src)
 {
-	HklGeometry *self = NULL;
+	HklGeometry *self = g_new(HklGeometry, 1);
 	HklParameter **axis;
 	HklHolder **holder;
 
-	self = HKL_MALLOC(HklGeometry);
-
 	*self = *src;
 
 	/* copy the axes */
@@ -667,15 +663,15 @@ int hkl_geometry_axis_values_set(HklGeometry *self,
 	HklParameter **axis;
 
 	hkl_error (error == NULL || *error == NULL);
-	g_assert(n_values == darray_size(self->axes));
+	hkl_assert(n_values == darray_size(self->axes));
 
 	darray_foreach(axis, self->axes){
 		if(!hkl_parameter_value_set(*axis, values[i++], unit_type, error)){
-			g_assert (error == NULL || *error != NULL);
+			hkl_assert (error == NULL || *error != NULL);
 			return FALSE;
 		}
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (error == NULL || *error == NULL);
 
 	hkl_geometry_update(self);
 
@@ -721,13 +717,13 @@ int hkl_geometry_set_values_v(HklGeometry *self, HklUnitEnum unit_type, GError *
 		if(!hkl_parameter_value_set(*axis,
 					    va_arg(ap, double),
 					    unit_type, error)){
-			g_assert (error == NULL || *error != NULL);
+			hkl_assert (error == NULL || *error != NULL);
 			va_end(ap);
 			hkl_geometry_update(self);
 			return FALSE;
 		}
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (error == NULL || *error == NULL);
 
 	va_end(ap);
 
@@ -857,9 +853,9 @@ int hkl_geometry_closest_from_geometry_with_range(HklGeometry *self,
 	}
 	if(!ko){
 		for(i=0;i<len;++i)
-			hkl_parameter_value_set(darray_item(self->axes, i),
-						values[i],
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(darray_item(self->axes, i),
+                                                       values[i],
+                                                       HKL_UNIT_DEFAULT, NULL));
 		hkl_geometry_update(self);
 	}
 	return ko;
@@ -952,9 +948,7 @@ HklVector hkl_geometry_kf_get(const HklGeometry *self,
  **/
 HklGeometryList *hkl_geometry_list_new(void)
 {
-	HklGeometryList *self;
-
-	self = HKL_MALLOC(HklGeometryList);
+	HklGeometryList *self = g_new(HklGeometryList, 1);
 
 	list_head_init(&self->items);
 	self->n_items = 0;
@@ -979,7 +973,7 @@ HklGeometryList *hkl_geometry_list_new_copy(const HklGeometryList *self)
 	if (!self)
 		return NULL;
 
-	dup = HKL_MALLOC(HklGeometryList);
+	dup = g_new(HklGeometryList, 1);
 
 	list_head_init(&dup->items);
 	/* now copy the item arrays */
@@ -1028,9 +1022,11 @@ void hkl_geometry_list_add(HklGeometryList *self, const HklGeometry *geometry)
 			return;
 	}
 
-	list_add_tail(&self->items,
-		      &hkl_geometry_list_item_new(geometry)->list);
-	self->n_items += 1;
+        item = hkl_geometry_list_item_new(geometry);
+        if (NULL != item){
+                list_add_tail(&self->items, &item->list);
+                self->n_items += 1;
+        }
 }
 
 /**
@@ -1158,6 +1154,10 @@ void hkl_geometry_list_fprintf(FILE *f, const HklGeometryList *self)
 		HklGeometryListItem *item;
 		HklParameter **axis;
 
+                item = list_top(&self->items, HklGeometryListItem, list);
+                if(!item)
+                        return;
+
 		fprintf(f, "    ");
 		darray_foreach(axis, list_top(&self->items, HklGeometryListItem, list)->geometry->axes){
 			fprintf(f, "%19s", (*axis)->name);
@@ -1269,6 +1269,9 @@ void hkl_geometry_list_multiply_from_range(HklGeometryList *self)
 		int *perm;
 		size_t j = 0;
 
+                if(!item)
+                        return;
+
 		geometry = hkl_geometry_new_copy(item->geometry);
 		perm = alloca(darray_size(geometry->axes) * sizeof(*perm));
 
@@ -1327,7 +1330,7 @@ HklGeometryListItem *hkl_geometry_list_item_new(const HklGeometry *geometry)
 	if(!geometry)
 		return NULL;
 
-	self = HKL_MALLOC(HklGeometryListItem);
+	self = g_new(HklGeometryListItem, 1);
 
 	self->geometry = hkl_geometry_new_copy(geometry);
 
@@ -1349,7 +1352,7 @@ HklGeometryListItem *hkl_geometry_list_item_new_copy(const HklGeometryListItem *
 	if(!self)
 		return NULL;
 
-	dup = HKL_MALLOC(HklGeometryListItem);
+	dup = g_new(HklGeometryListItem, 1);
 
 	dup->geometry = hkl_geometry_new_copy(self->geometry);
 
