@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2019 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019, 2022 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -93,12 +93,12 @@ HklLattice *hkl_lattice_new(double a, double b, double c,
 
 	if(!check_lattice_param(a, b, c, alpha, beta, gamma, &volume, error))
 	{
-		g_assert (error == NULL || *error != NULL);
+		hkl_assert (error == NULL || *error != NULL);
 		return FALSE;
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (error == NULL || *error == NULL);
 
-	self = HKL_MALLOC(HklLattice);
+	self = g_new(HklLattice, 1);
 
 	self->a = hkl_parameter_new("a", "The length of the first lattice vector",
 				    0, a, a+10,
@@ -153,9 +153,7 @@ HklLattice *hkl_lattice_new(double a, double b, double c,
  **/
 HklLattice *hkl_lattice_new_copy(const HklLattice *self)
 {
-	HklLattice *copy = NULL;
-
-	copy = HKL_MALLOC(HklLattice);
+	HklLattice *copy = g_new(HklLattice, 1);
 
 	copy->a = hkl_parameter_new_copy(self->a);
 	copy->b = hkl_parameter_new_copy(self->b);
@@ -208,10 +206,10 @@ void hkl_lattice_free(HklLattice *self)
 		_p = hkl_parameter_value_get((_parameter), HKL_UNIT_DEFAULT); \
 									\
 		if(!hkl_lattice_set(self, a, b, c, alpha, beta, gamma, HKL_UNIT_DEFAULT, (_error))){ \
-			g_assert ((_error) == NULL || *(_error) != NULL); \
+			hkl_assert ((_error) == NULL || *(_error) != NULL); \
 			return FALSE;					\
 		}							\
-		g_assert ((_error) == NULL || *(_error) == NULL);	\
+		hkl_assert ((_error) == NULL || *(_error) == NULL);	\
 		return hkl_parameter_init_copy(self->_p, (_parameter), (_error)); \
 	}while(0)
 
@@ -414,19 +412,19 @@ int hkl_lattice_set(HklLattice *self,
 
 	/* need to do the conversion before the check */
 	if(!check_lattice_param(_a, _b, _c, _alpha, _beta, _gamma, &_volume, error)){
-		g_assert (error == NULL || *error != NULL);
+		hkl_assert (error == NULL || *error != NULL);
 		return FALSE;
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (error == NULL || *error == NULL);
 
-	hkl_parameter_value_set(self->a, _a, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->b, _b, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->c, _c, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->alpha, _alpha, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->beta, _beta, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->gamma, _gamma, HKL_UNIT_DEFAULT, NULL);
+	IGNORE(hkl_parameter_value_set(self->a, _a, HKL_UNIT_DEFAULT, NULL));
+	IGNORE(hkl_parameter_value_set(self->b, _b, HKL_UNIT_DEFAULT, NULL));
+	IGNORE(hkl_parameter_value_set(self->c, _c, HKL_UNIT_DEFAULT, NULL));
+	IGNORE(hkl_parameter_value_set(self->alpha, _alpha, HKL_UNIT_DEFAULT, NULL));
+	IGNORE(hkl_parameter_value_set(self->beta, _beta, HKL_UNIT_DEFAULT, NULL));
+	IGNORE(hkl_parameter_value_set(self->gamma, _gamma, HKL_UNIT_DEFAULT, NULL));
 
-	hkl_parameter_value_set(self->volume, _volume, HKL_UNIT_DEFAULT, NULL);
+	IGNORE(hkl_parameter_value_set(self->volume, _volume, HKL_UNIT_DEFAULT, NULL));
 
 	return TRUE;
 }
@@ -606,14 +604,14 @@ int hkl_lattice_reciprocal(const HklLattice *self, HklLattice *reciprocal)
 	s_beta2 = D / s_gamma_s_alpha;
 	s_beta3 = D / s_alpha_s_beta;
 
-	hkl_lattice_set(reciprocal,
-			HKL_TAU * s_alpha / (hkl_parameter_value_get(self->a, HKL_UNIT_DEFAULT) * D),
-			HKL_TAU * s_beta  / (hkl_parameter_value_get(self->b, HKL_UNIT_DEFAULT) * D),
-			HKL_TAU * s_gamma / (hkl_parameter_value_get(self->c, HKL_UNIT_DEFAULT) * D),
-			atan2(s_beta1, c_beta1),
-			atan2(s_beta2, c_beta2),
-			atan2(s_beta3, c_beta3),
-			HKL_UNIT_DEFAULT, NULL);
+	IGNORE(hkl_lattice_set(reciprocal,
+                               HKL_TAU * s_alpha / (hkl_parameter_value_get(self->a, HKL_UNIT_DEFAULT) * D),
+                               HKL_TAU * s_beta  / (hkl_parameter_value_get(self->b, HKL_UNIT_DEFAULT) * D),
+                               HKL_TAU * s_gamma / (hkl_parameter_value_get(self->c, HKL_UNIT_DEFAULT) * D),
+                               atan2(s_beta1, c_beta1),
+                               atan2(s_beta2, c_beta2),
+                               atan2(s_beta3, c_beta3),
+                               HKL_UNIT_DEFAULT, NULL));
 
 	return TRUE;
 }
@@ -661,8 +659,8 @@ void hkl_lattice_randomize(HklLattice *self)
 										 HKL_UNIT_DEFAULT));
 
 			/* compute the alpha angle. */
-			hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
+                                                       HKL_UNIT_DEFAULT, NULL));
 		} else if (self->beta->fit) {
 			/* beta */
 			a = b = vector_x;
@@ -681,8 +679,8 @@ void hkl_lattice_randomize(HklLattice *self)
 										 HKL_UNIT_DEFAULT));
 
 			/* compute beta */
-			hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
+                                                       HKL_UNIT_DEFAULT, NULL));
 		} else {
 			/* gamma */
 			a = c = vector_x;
@@ -701,8 +699,8 @@ void hkl_lattice_randomize(HklLattice *self)
 										 HKL_UNIT_DEFAULT));
 
 			/* compute gamma */
-			hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
+                                                       HKL_UNIT_DEFAULT, NULL));
 		}
 		break;
 	case 2:
@@ -720,10 +718,10 @@ void hkl_lattice_randomize(HklLattice *self)
 				/* randomize c */
 				hkl_vector_randomize_vector_vector(&c, &a, &b);
 
-				hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
-							HKL_UNIT_DEFAULT, NULL);
-				hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
-							HKL_UNIT_DEFAULT, NULL);
+				IGNORE(hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
+                                                               HKL_UNIT_DEFAULT, NULL));
+				IGNORE(hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
+                                                               HKL_UNIT_DEFAULT, NULL));
 			} else {
 				/* alpha + gamma */
 				a = c = vector_x;
@@ -737,10 +735,10 @@ void hkl_lattice_randomize(HklLattice *self)
 				/* randomize c */
 				hkl_vector_randomize_vector_vector(&b, &a, &c);
 
-				hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
-							HKL_UNIT_DEFAULT, NULL);
-				hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
-							HKL_UNIT_DEFAULT, NULL);
+				IGNORE(hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
+                                                               HKL_UNIT_DEFAULT, NULL));
+				IGNORE(hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
+                                                               HKL_UNIT_DEFAULT, NULL));
 			}
 		} else {
 			/* beta + gamma */
@@ -755,10 +753,10 @@ void hkl_lattice_randomize(HklLattice *self)
 			/* randomize c */
 			hkl_vector_randomize_vector_vector(&a, &b, &c);
 
-			hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
-						HKL_UNIT_DEFAULT, NULL);
-			hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
+                                                       HKL_UNIT_DEFAULT, NULL));
+			IGNORE(hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
+                                                       HKL_UNIT_DEFAULT, NULL));
 		}
 		break;
 	case 3:
@@ -766,12 +764,12 @@ void hkl_lattice_randomize(HklLattice *self)
 		hkl_vector_randomize_vector(&b, &a);
 		hkl_vector_randomize_vector_vector(&c, &b, &a);
 
-		hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
-					HKL_UNIT_DEFAULT, NULL);
-		hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
-					HKL_UNIT_DEFAULT, NULL);
-		hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
-					HKL_UNIT_DEFAULT, NULL);
+		IGNORE(hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
+                                               HKL_UNIT_DEFAULT, NULL));
+		IGNORE(hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
+                                               HKL_UNIT_DEFAULT, NULL));
+		IGNORE(hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
+                                               HKL_UNIT_DEFAULT, NULL));
 		break;
 	}
 }
